/**
 * Tonjoo Responsive Slideshow
 * Copyright 2013, Tonjoo
 * Free to use under the MIT license.
 * http://www.opensource.org/licenses/mit-license.php
 */

;(function($) {

    $.TonjooResponsiveSlideshow = function(el, opt) {

        var base = this,
            opt;

        base.el = el;
        base.$el = $(base.el);

        base.activeSlide = 0;
        base.activeSlideContinous = 0;
        base.numberSlides = 0;
        base.continous_count_position = 0;
        base.trsId = "#" + base.$el.attr("id");        

        /**
         * Function: initiate
         */
        base.initialize = function() 
        {
            base.$slideWrapper = base.$el.children('.trs-slide-img-wrapper').addClass('trs-slide-img-wrapper');
            base.$trs = base.$slideWrapper.wrap('<div class="trs-slideshow-content" />').parent();
            base.$trsWrapper = base.$trs.wrap('<div id="' + base.trsId + '-slideshow" class="trs-wrapper ' + opt.skinClass.toLowerCase() + '" />').parent();
            
            base.first_run = true;
            base.old_responsive_class = 'responsive-full';
            base.responsiveClassLock = false;

            base.lock(); // Lock slider before all content loaded

            // Initialize and show slider after all content loaded
            $(base.$slideWrapper.children()).imagesLoaded( function() {
                base.$slideWrapper.fadeIn(function(){
                    base.$el.css({"display": "block"});
                })

                base.$slideWrapper.children().fadeIn(function(){
                    base.$el.css({"display": "block"});
                })
                
                base.$trsWrapper.children('.trs-slideshow-content').fadeIn(function(){
                    base.$el.css({"display": "block"});
                })

                base.$trsWrapper.children('.trs-timer').fadeIn(function(){
                    base.$el.css({"display": "block"});
                })

                base.$trsWrapper.children('.trs-slider-nav').fadeIn(function(){
                    base.$el.css({"display": "block"});
                })

                base.$trsWrapper.children('.trs-pagination-wrapper').fadeIn(function(){
                    base.$el.css({"display": "block"});

                    //unlock event in last displayed element
                    base.unlock();
                })
            })

            base.$trs.css({
                'height': opt.height,
                'max-width': opt.width
            });

            base.$trsWrapper.parent().css({
                'height': opt.height,
                'max-width': opt.width
            });

            base.$trs.add(base.trsWidth)

            // Initialize slides
            base.$slides = base.$slideWrapper.children('div.trs-slide-img');

            base.$slides.each(function (index,slide) {
                base.numberSlides++;
                base.activeSlideContinous++;
            });

            base.setupLayout();
            base.setupTimer();
            base.setupDirectionalNav();
            
            base.bulletObj = new base.setupSliderBulletNav();

            base.setupBulletNav();

            base.setCaptionPosition();
            base.setupSwipeTouch();
            base.resetSlider();

            $(window).bind('resize.trs-slideshow-container', function(event, force){                
                base.resetSlider();             
            });
        }

        /**
         * Function: unlock
         */
        base.unlock = function()
        {
            base.locked = false;
        }

        /**
         * Function: lock
         */
        base.lock = function()
        {
            base.locked = true;
        }

        /**
         * Function: calculateHeightWidth
         */
        base.calculateHeightWidth = function()
        {
            base.trsWidth = base.$trs.innerWidth();

            base.subSlideWidth = base.numberSlides * base.trsWidth;
            base.subSlideHeight = base.numberSlides * base.trsHeight;

            var minusResize = opt.width - base.trsWidth;
            var percentMinus = (minusResize / opt.width) * 100;
            base.trsHeight = opt.height - (opt.height * percentMinus / 100);
        }

        /**
         * Function: css3support
         */
        base.css3support = function()
        {
            var element = document.createElement('div'),
                props = [ 'perspectiveProperty', 'WebkitPerspective', 'MozPerspective', 'OPerspective', 'msPerspective' ];
            
            for ( var i in props ) {
                if ( typeof element.style[ props[ i ] ] !== 'undefined' ) {
                    base.vendorPrefix = props[i].replace('Perspective', '').toLowerCase();
                    return opt.jsOnly ? false : true;
                }
            }

            return false;
        }

        /**
         * Function: setupLayout
         */
        base.setupLayout = function()
        {
            base.calculateHeightWidth();

            if(opt.animation == "horizontal-slide")
            {
                base.$slides.css({
                    "position": "relative",
                    "float": "left",
                    "display": "block",
                    "width": base.trsWidth + "px",
                    "height": + "100%"
                });

                if(opt.continousSliding)
                {
                    base.$slideWrapper.css({"width": base.trsWidth * base.numberSlides * 3 + "px", "height": base.trsHeight + "px"});
                    base.slideWrapperInside = '<div class="slideWrapperInside" style="float:left;">' + base.$slideWrapper.html() + '</div>';
                    base.$slideWrapper.html(base.slideWrapperInside + base.slideWrapperInside + base.slideWrapperInside);
                }
                else
                {  
                    base.$slideWrapper.css({"width": base.trsWidth * base.numberSlides + "px", "height": base.trsHeight + "px"});
                }
            }
            else if(opt.animation == "vertical-slide")
            {
                base.$slides.css({
                    "position": "relative",
                    "display": "block",
                    "width": base.trsWidth + "px",
                    "height": base.trsHeight + "px"
                });

                if(opt.continousSliding)
                {
                    base.$slideWrapper.css({"width": base.trsWidth + "px", "height": base.trsHeight * base.numberSlides * 3 + "px"});
                    base.slideWrapperInside = '<div class="slideWrapperInside">' + base.$slideWrapper.html() + '</div>';
                    base.$slideWrapper.html(base.slideWrapperInside + base.slideWrapperInside + base.slideWrapperInside);
                }
                else
                {
                    base.$slideWrapper.css({"width": base.trsWidth + "px", "height": base.trsHeight * base.numberSlides + "px"});
                }
            }
            else if(opt.animation == "fade")
            {
                base.$slides.css({
                    "z-index": 1,
                    "width": base.trsWidth + "px",
                    "height": base.trsHeight + "px"
                });

                base.$slides.eq(base.activeSlide).css({"z-index": 3});
                base.$slideWrapper.css({"width": base.trsWidth + "px", "height": base.trsHeight + "px"});
            }
        }

        /**
         * Function: setupTimer
         */
        base.setupTimer = function()
        {
            date = new Date();
            milliseconds = date.getTime();
            startSeconds = milliseconds / 1000;

            function log_time() {
                date = new Date();
                milliseconds = date.getTime();
                seconds = milliseconds / 1000;
                seconds = seconds - startSeconds;         
            }

            //Timer Execution
            function startClock() {
                if (!opt.timer || opt.timer == 'false') {
                    return false;
                                
                /**
                 * Because in startup timer is always hidden
                 * use this if you want to change the behaviour
                 *
                 * } else if (timer.is(':hidden')) {
                 *       base.timerRunning = true;
                 *       base.clock = setInterval(function (e) {
                 *
                 *           shift("next");
                 *
                 *      }, opt.advanceSpeed);
                 *
                 */

                } else if(! base.css3support()) {
                    base.timerRunning = true;
                    base.$pause.removeClass('trs-timer-active');
                    base.clock = setInterval(function (e) {
                
                        base.shift("next", true);
                
                    }, opt.advanceSpeed);
                } else {
                    base.timerRunning = true;
                    base.$pause.removeClass('trs-timer-active');
                    base.clock = setInterval(function (e) {

                        var degreeCSS = "rotate(" + degrees + "deg)"
                        rotator.css('-' + base.vendorPrefix + '-transform', degreeCSS);
                        degrees += 1
                        if (degrees >= 180) {

                            mask.addClass('trs-timer-move')
                            rotator.addClass('trs-timer-move')
                            mask_turn.css("display", "block")

                        }
                        if (degrees >= 360) {

                            degrees = 0;
                            mask.removeClass('trs-timer-move')
                            rotator.removeClass('trs-timer-move')
                            mask_turn.css("display", "none")

                            base.shift("next", true);
                        }
                    }, opt.advanceSpeed / 360);
                }
            }

            // Timer Setup
            if (opt.timer) {
                var timerHTML = '<div class="trs-timer"><span class="trs-timer-mask"><span class="trs-timer-rotator"></span></span><span class="trs-timer-mask-turn"></span><span class="trs-timer-pause"></span></div>';
                
                base.$trsWrapper.append(timerHTML);

                var timer = base.$trsWrapper.children('div.trs-timer');

                if (timer.length != 0) {
                    var rotator = $(base.trsId + ' div.trs-timer span.trs-timer-rotator'),
                        mask = $(base.trsId + ' div.trs-timer span.trs-timer-mask'),
                        mask_turn = $(base.trsId + ' div.trs-timer span.trs-timer-mask-turn'),
                        degrees = 0;

                    base.$pause = $(base.trsId + ' div.trs-timer span.trs-timer-pause')

                    startClock();
                    timer.click(function () {
                        if (!base.timerRunning) {
                            startClock();
                        } else {
                            base.stopSliderLock();
                        }
                    });
                    if (opt.startClockOnMouseOut) {
                        var outTimer;
                        base.$trsWrapper.mouseleave(function () {

                            outTimer = setTimeout(function () {
                                if (!base.timerRunning) {
                                    startClock();
                                }
                            }, opt.startClockOnMouseOutAfter)
                        })
                        base.$trsWrapper.mouseenter(function () {
                            clearTimeout(outTimer);
                        })
                    }
                }
            }

            // Pause Timer on hover
            if (opt.pauseOnHover) {
                base.$trsWrapper.mouseenter(function () {
                    base.stopSliderLock();
                });
            }
        }

        /**
         * Function: stopSliderLock
         */
        base.stopSliderLock = function()
        {
            if (!opt.timer || opt.timer == 'false') {
                return false;
            } else {
                base.timerRunning = false;
                clearInterval(base.clock);

                base.$pause.addClass('trs-timer-active');
            }
        }

        /**
         * Function: setupDirectionalNav
         */
        base.setupDirectionalNav = function()
        {
            if (opt.directionalNav != 'none') 
            {
                if (opt.directionalNav == "false") {
                    return false;
                }
                var directionalNavHTML = '<div class="trs-slider-nav ' + base.captionPosition + '"><span class="trs-arrow-right"></span><span class="trs-arrow-left"></span></div>';
                base.$trsWrapper.append(directionalNavHTML);
                var leftBtn = base.$trsWrapper.children('div.trs-slider-nav').children('span.trs-arrow-left'),
                    rightBtn = base.$trsWrapper.children('div.trs-slider-nav').children('span.trs-arrow-right');
                leftBtn.click(function () {
                    base.stopSliderLock();
                    base.shift("prev", true);
                });
                rightBtn.click(function () {
                    base.stopSliderLock();
                    base.shift("next", true)
                });

                // autohide behaviour
                if(opt.directionalNav == 'autohide')
                {
                    var btn = base.$trsWrapper.children('div.trs-slider-nav').children('span');
                    var btnAnimateSpeed = 300;

                    btn.css("opacity", opt.directionalNavHideOpacity);

                    base.$trsWrapper.mouseenter(function(){
                        btn.animate({
                            "opacity": opt.directionalNavShowOpacity
                        }, btnAnimateSpeed);
                    });
                    base.$trsWrapper.mouseleave(function(){
                        btn.animate({
                            "opacity": opt.directionalNavHideOpacity
                        }, btnAnimateSpeed);
                    });
                }                
            }
        }

        /**
         * Function: setupBulletNav
         */
        base.setupBulletNav = function()
        {
            var bulletHTML = '<ul class="trs-pagination trs-pagination-' + opt.pagination + '"></ul>';

            var bulletHTMLWrapper = "<div class='trs-bullet-wrapper'></div>";
            
            base.$trsWrapper.append(bulletHTML);
            base.$pagination = base.$trsWrapper.children('ul.trs-pagination');

            for (i = 0; i < base.numberSlides; i++) 
            {
                var liMarkup = jQuery('<li class="trs-slideshow-nav-pagination"></li>');

                if (opt.pagination == 'text' || opt.pagination == 'image') 
                {
                    var paginationContent = opt.paginationContent.length > 0 ? opt.paginationContent[i] : "";
                    var liMarkup = $('<li class="trs-slideshow-nav-pagination">' + paginationContent + '</li>');
                }

                base.$trsWrapper.children('ul.trs-pagination').append(liMarkup);
                liMarkup.data('index', i);
                liMarkup.click(function () {                        
                    base.stopSliderLock();
                    base.shift($(this).data('index'), true);
                });
            }
           
            base.$pagination.wrap("<div class='trs-pagination-wrapper " + base.captionPosition + "' />");                              
            base.bulletObj.setActiveBullet();

            // if (opt.pagination == 'none')
            // {                
            //     base.$pagination.css('display','none');
            // }
        }

        /**
         * Function: setupSliderBulletNav
         */
        base.setupSliderBulletNav = function()
        {
            var spagination = 0;
            var paginationWalkingWidth = 0;            
            var paginationMaxShowedIndex = 0;
            var paginationBackChild = 0;
            var paginationNextChild = 0;
            var paginationOffsetWidth = 0;
            var paginationPosition = 0;                        
            var paginationOffsetEnable = false;
            var paginationWidth = 0;
            var paginationMovedWidth = 0;

            var each_width = opt.paginationContentWidth;
            var total_width = each_width * base.numberSlides;
            
            /** 
             * generate slide bullet 
             * this function will be recall every slideshow resized
             */
            this.generateSlideBullet = function()
            {
                spagination = base.$trsWrapper.find('ul.trs-pagination-text');

                paginationWalkingWidth = 0;
                paginationMaxShowedIndex = 0;
                paginationBackChild = 0;
                paginationNextChild = 0;
                paginationOffsetWidth = 0;
                paginationPosition = 0;                        
                paginationOffsetEnable = false;              
                paginationWidth = spagination.parent().outerWidth(true);

                if(paginationWidth > total_width)
                {
                    each_width = paginationWidth / base.numberSlides;
                    total_width = each_width * base.numberSlides;
                }
                                
                spagination.parent().css('overflow', 'hidden');
                spagination.css('background-color', spagination.children('li').last().css("background-color"));
                spagination.children('li.trs-slideshow-nav-pagination').css('width',each_width + 'px');                
                spagination.css('width', total_width + 'px');

                spagination.find('li').each(function () {
                    paginationWalkingWidth += each_width;

                    if (paginationWalkingWidth + each_width > paginationWidth) 
                    {
                        paginationNextChild = $(this).index();
                        paginationMaxShowedIndex = paginationNextChild;
                    }
                    
                    if (paginationWalkingWidth > paginationWidth) 
                    {
                        $(this).addClass('trs-bullet-sliding-next');
                        paginationOffsetWidth = paginationWalkingWidth - paginationWidth;

                        /* detect if pagination offset is too large */
                        if(paginationOffsetWidth < each_width)
                        {
                            paginationOffsetEnable = true;
                        }

                        return false;
                    }
                });
            }

            this.slideBullet = function(navigate)
            {
                if(navigate == 'next')
                {
                    if(spagination.children('li').eq(base.numberSlides - 1).hasClass("trs-bullet-sliding-next"))
                    {
                        paginationNavPixelWidth = (each_width * paginationPosition) + paginationOffsetWidth;
                    }
                    else
                    {
                        spagination.children('li').removeClass('trs-bullet-sliding-back').removeClass('trs-bullet-sliding-next');

                        paginationPosition++;
                        paginationBackChild++;
                        paginationNextChild++;

                        paginationNavPixelWidth = (each_width * paginationPosition) + paginationOffsetWidth;
                    }

                    slideBulletAddClass('sliding_one','trs-bullet-sliding-one-back',paginationBackChild + 1)            
                }
                else if(navigate == 'back')
                {
                    spagination.children('li').removeClass('trs-bullet-sliding-back').removeClass('trs-bullet-sliding-next');

                    paginationPosition--;
                    paginationBackChild--;
                    paginationNextChild--;

                    paginationNavPixelWidth = each_width * paginationPosition;

                    slideBulletAddClass('sliding_one','trs-bullet-sliding-one-next',paginationNextChild - 1)
                }
                else if(navigate == 'first')
                {
                    spagination.children('li').removeClass('trs-bullet-sliding-back').removeClass('trs-bullet-sliding-next');

                    paginationPosition = 0;
                    paginationBackChild = 0;                    
                    paginationNextChild = paginationMaxShowedIndex;

                    paginationNavPixelWidth = each_width * paginationPosition;

                    slideBulletAddClass('sliding_one','trs-bullet-sliding-one-next',paginationNextChild - 1)
                }
                else if(navigate == 'last')
                {
                    spagination.children('li').removeClass('trs-bullet-sliding-back').removeClass('trs-bullet-sliding-next');

                    var numberBulletsByIndex = base.numberSlides - 1;

                    paginationPosition = numberBulletsByIndex - paginationMaxShowedIndex;
                    paginationBackChild = numberBulletsByIndex - paginationMaxShowedIndex;
                    paginationNextChild = numberBulletsByIndex;

                    paginationNavPixelWidth = (each_width * paginationPosition) + paginationOffsetWidth;

                    slideBulletAddClass('sliding_one','trs-bullet-sliding-one-back',paginationBackChild + 1)
                }

                /**
                 * Track moved width
                 */
                paginationMovedWidth = paginationNavPixelWidth;
                
                if(base.css3support())
                {
                    var properties = {};
                    properties[ '-' + base.vendorPrefix + '-transition-duration' ] = opt.animationSpeed + 'ms';
                    properties[ '-' + base.vendorPrefix + '-transform' ] = 'translate3d(-' + paginationNavPixelWidth + 'px, 0, 0)';

                    spagination.css(properties);
                }
                else
                {
                    spagination
                        .animate({
                            "left": '-' + paginationNavPixelWidth + 'px'
                        }, opt.animationSpeed);
                }
                
                // Apply class to bullet
                slideBulletAddClass('sliding', 'trs-bullet-sliding-back', paginationBackChild)
                slideBulletAddClass('sliding', 'trs-bullet-sliding-next', paginationNextChild)
            }

            function slideBulletAddClass(li_type, li_class, li_index)
            {
                if(li_type == 'sliding_one')
                {
                    if(paginationOffsetEnable == true)
                    {
                        spagination.children('li').removeClass('trs-bullet-sliding-one-back').removeClass('trs-bullet-sliding-one-next');
                        
                        var addClassTo = spagination.children('li').eq(li_index)

                        if(addClassTo.attr('class') == "trs-slideshow-nav-pagination")
                        {
                            addClassTo.addClass(li_class);
                        }                        
                    }
                }  
                else if(li_type == 'sliding')
                {
                    if(spagination.children('li').eq(li_index).attr('class') == 'trs-slideshow-nav-pagination trs-pagination-active' && li_index > 0 )
                    {
                        if(li_class == 'trs-bullet-sliding-back')
                        {
                            li_index--
                        }
                        else if(li_class == 'trs-bullet-sliding-next')
                        {
                            li_index++
                        }
                    }

                    spagination.children('li').eq(li_index).removeClass('trs-bullet-sliding-one-back')
                    spagination.children('li').eq(li_index).removeClass('trs-bullet-sliding-one-next')
                    spagination.children('li').eq(li_index).addClass(li_class)
                }              
            }

            function slideBulletOne(type)
            {
                var oneMove = 0;

                spagination.children('li').removeClass('trs-bullet-sliding-one-back').removeClass('trs-bullet-sliding-one-next');

                if(type == 'back')
                {
                    spagination.children('li').eq(paginationNextChild - 1).addClass('trs-bullet-sliding-one-next');

                    oneMove = paginationMovedWidth - paginationOffsetWidth;
                }
                else
                {
                    spagination.children('li').eq(paginationBackChild + 1).addClass('trs-bullet-sliding-one-back');

                    oneMove = paginationMovedWidth + paginationOffsetWidth;
                }

                /**
                 * Track moved width
                 */
                paginationMovedWidth = oneMove;

                if(base.css3support())
                {
                    var properties = {};
                    properties[ '-' + base.vendorPrefix + '-transition-duration' ] = opt.animationSpeed + 'ms';
                    properties[ '-' + base.vendorPrefix + '-transform' ] = 'translate3d(-' + oneMove + 'px, 0, 0)';

                    spagination.css(properties);
                }
                else
                {
                    spagination
                        .animate({
                            "left": '-' + oneMove + 'px'
                        }, opt.animationSpeed);
                }
            }


            /**
             * SET ACTIVE BULLETS
             */
            this.setActiveBullet = function() 
            {
                base.beforeSlideChange(); // before slide function

                if (opt.pagination == 'none') {
                    return false;
                } else {
                    base.$pagination.children('li').removeClass('trs-pagination-active').eq(base.activeSlide).addClass('trs-pagination-active');

                    /**
                     * begin slide pagination
                     */
                    if(opt.pagination == 'text' || opt.pagination == 'image') this.beginSlideBullet();
                }
            }

            this.beginSlideBullet = function()
            {
                var activeBullet = base.$pagination.children('li.trs-pagination-active');
                
                var positionFirst = activeBullet.index() == 0 ? true : false;
                var positionMiddle = activeBullet.index() > 0 ? true : false;
                var positionLast = activeBullet.index() == (base.numberSlides-1) ? true : false;

                if(activeBullet.hasClass('trs-bullet-sliding-next'))
                {
                    if(positionLast) this.slideBullet('last');
                    else this.slideBullet('next');
                }
                else if(activeBullet.hasClass('trs-bullet-sliding-back'))
                {
                    if(positionMiddle) this.slideBullet('back');
                    else this.slideBullet('first');
                }
                else if(activeBullet.hasClass('trs-bullet-sliding-one-next'))
                {
                    slideBulletOne('next')
                }
                else if(activeBullet.hasClass('trs-bullet-sliding-one-back'))
                {
                    slideBulletOne('back')
                }
                else
                {
                    if(paginationMaxShowedIndex > 0)
                    {
                        if(positionFirst) this.slideBullet('first');
                        else if(positionLast) this.slideBullet('last');
                    }
                }
            }
        }

        /**
         * Function: setCaptionPosition
         */
        base.setCaptionPosition = function()
        {
            caption = base.$slides.eq(base.activeSlide).find('div.trs-caption');

            if(caption.length)
            {
                base.captionPosition = caption.attr('class').replace('trs-caption ','');
            }
            else
            {
                base.captionPosition = "undefined";
            }

            //set active caption position to bullet and navigation
            base.$trsWrapper.find('div.trs-pagination-wrapper').attr('class', 'trs-pagination-wrapper ' + base.captionPosition);
            base.$trsWrapper.find('div.trs-slider-nav').attr('class', 'trs-slider-nav ' + base.captionPosition);
        }

        /**
         * Function: resetAndUnlock
         */
        base.resetAndUnlock = function(timeout)
        {
            if(timeout)
            {
                clearTimeout(timeout);
                timeout = setTimeout(function() {
                    base.unlock();
                    base.afterSlideChange();
                }, opt.animationSpeed - (opt.animationSpeed * 20 / 100));
            }
            else
            {
                base.unlock();
                base.afterSlideChange();
            }
        }

        /**
         * Function: afterSlideChange
         */
        base.beforeSlideChange = function()
        {
            var paginationClass = opt.paginationExternalClass;

            if(paginationClass != "" && $('.' + paginationClass).length)
            {
                $("." + paginationClass).removeClass('active');
                $("." + paginationClass).eq(base.activeSlide).addClass("active");
            }
        }

        /**
         * Function: afterSlideChange
         */
        base.afterSlideChange = function()
        {
            // empty function
        }

        /**
         * Function: shift
         */
        base.shift = function(direction, doAnimation)
        {
            // remember previous activeSlide
            var prevActiveSlide = base.activeSlide,
                slideDirection = direction;

            // exit function if bullet clicked is same as the current image
            if (prevActiveSlide == slideDirection) {
                return false;
            }

            if (! base.locked) 
            {
                base.lock();
                //deduce the proper activeImage
                if (direction == "next") 
                {
                    base.activeSlide++
                    base.activeSlideContinous++
                    if (base.activeSlide == base.numberSlides) {
                        base.activeSlide = 0;
                    }
                } 
                else if (direction == "prev") 
                {
                    base.activeSlide--
                    base.activeSlideContinous--
                    if (base.activeSlide < 0) {
                        base.activeSlide = base.numberSlides - 1;
                    }
                } 
                else 
                {
                    var mod_position = base.activeSlideContinous % base.numberSlides;
                    var float_position = base.activeSlideContinous - mod_position;

                    for (var i = 0; i < direction; i++) 
                    {
                        float_position++

                        if(i == direction)
                        {
                            break;
                        }
                    }

                    // for minus position (back)
                    if(base.activeSlideContinous < 0 && mod_position != 0)
                    {
                        float_position = float_position - base.numberSlides;
                    }

                    base.activeSlideContinous = float_position;
                    base.activeSlide = direction;

                    if (prevActiveSlide < base.activeSlide) 
                    {
                        slideDirection = "next";
                    } 
                    else if (prevActiveSlide > base.activeSlide) 
                    {
                        slideDirection = "prev"
                    }
                }

                // set to correct bullet
                base.bulletObj.setActiveBullet();

                base.calculateHeightWidth();

                
                /**
                 * Horizontal Slide
                 */
                if (opt.animation == "horizontal-slide") 
                {
                    if(opt.continousSliding)
                    {
                        if(base.activeSlideContinous > 0)
                        {
                            var slide_action = '-' + base.trsWidth * base.activeSlideContinous;
                        }
                        else if(base.activeSlideContinous < 0)
                        {
                            var slide_action = base.trsWidth * base.activeSlideContinous * -1;
                        }
                        else
                        {
                            var slide_action = 0;
                        }


                        var slide_width_position = base.trsWidth * base.activeSlideContinous - (base.subSlideWidth * base.continous_count_position);
                        

                        if(slide_width_position < base.subSlideWidth)
                        {
                            base.continous_count_position--;
                           
                            base.$slideWrapper.addClass('notransition'); // Disable transitions

                            base.$slideWrapper.css('margin-left', (base.subSlideWidth * base.continous_count_position) + 'px');

                            base.$slideWrapper[0].offsetHeight; // Trigger a reflow, flushing the CSS changes
                            base.$slideWrapper.removeClass('notransition'); // Re-enable transitions
                        }
                        else if(slide_width_position >= base.subSlideWidth * 2)
                        {
                            base.continous_count_position++;
                           
                            base.$slideWrapper.addClass('notransition'); // Disable transitions

                            base.$slideWrapper.css('margin-left', (base.subSlideWidth * base.continous_count_position) + 'px');

                            base.$slideWrapper[0].offsetHeight; // Trigger a reflow, flushing the CSS changes
                            base.$slideWrapper.removeClass('notransition'); // Re-enable transitions
                        }

                        if(doAnimation)
                        {
                            if(base.css3support())
                            {
                                var properties = {};
                                properties[ '-' + base.vendorPrefix + '-transition-duration' ] = opt.animationSpeed + 'ms';
                                properties[ '-' + base.vendorPrefix + '-transform' ] = 'translate3d('+ slide_action +'px, 0, 0)';

                                // Do the CSS3 transition
                                base.$slideWrapper.css(properties);

                                base.resetAndUnlock(true);
                            }
                            else
                            {
                                base.$slideWrapper
                                .animate({
                                    "left": slide_action + 'px'
                                }, opt.animationSpeed, base.resetAndUnlock);
                            } 
                        }
                        else
                        {
                            base.resetAndUnlock();
                        }
                    }
                    else
                    {
                        var slide_action = base.trsWidth * base.activeSlide < base.numberSlides * base.trsWidth ? '-' + base.trsWidth * base.activeSlide : 0 ;

                        if(doAnimation)
                        {
                            if(base.css3support())
                            {
                                // Get the properties to transition
                                var properties = {};
                                properties[ '-' + base.vendorPrefix + '-transition-duration' ] = opt.animationSpeed + 'ms';
                                properties[ '-' + base.vendorPrefix + '-transform' ] = 'translate3d('+ slide_action +'px, 0, 0)';

                                // Do the CSS3 transition
                                base.$slideWrapper.css(properties);
                                
                                base.resetAndUnlock(true);
                            }
                            else
                            {
                                base.$slideWrapper
                                    .animate({
                                        "left": slide_action + 'px'
                                    }, opt.animationSpeed, base.resetAndUnlock);
                            }
                        }
                        else
                        {
                            base.resetAndUnlock();
                        }
                    }
                }

                /**
                 * Vertical Slide
                 */
                if (opt.animation == "vertical-slide") 
                {
                    if(opt.continousSliding)
                    {
                        if(base.activeSlideContinous > 0)
                        {
                            var slide_action = '-' + base.trsHeight * base.activeSlideContinous;
                        }
                        else if(base.activeSlideContinous < 0)
                        {
                            var slide_action = base.trsHeight * base.activeSlideContinous * -1;
                        }
                        else
                        {
                            var slide_action = 0;
                        }


                        var slide_width_position = base.trsHeight * base.activeSlideContinous - (base.subSlideHeight * base.continous_count_position);
                        

                        if(slide_width_position < base.subSlideHeight)
                        {
                            base.continous_count_position--;
                           
                            base.$slideWrapper.addClass('notransition'); // Disable transitions

                            base.$slideWrapper.css('margin-top', (base.subSlideHeight * base.continous_count_position) + 'px');

                            base.$slideWrapper[0].offsetHeight; // Trigger a reflow, flushing the CSS changes
                            base.$slideWrapper.removeClass('notransition'); // Re-enable transitions
                        }
                        else if(slide_width_position >= base.subSlideHeight * 2)
                        {
                            base.continous_count_position++;
                           
                            base.$slideWrapper.addClass('notransition'); // Disable transitions

                            base.$slideWrapper.css('margin-top', (base.subSlideHeight * base.continous_count_position) + 'px');

                            base.$slideWrapper[0].offsetHeight; // Trigger a reflow, flushing the CSS changes
                            base.$slideWrapper.removeClass('notransition'); // Re-enable transitions
                        }

                        if(doAnimation)
                        {
                            if(base.css3support())
                            {
                                var properties = {};
                                properties[ '-' + base.vendorPrefix + '-transition-duration' ] = opt.animationSpeed + 'ms';
                                properties[ '-' + base.vendorPrefix + '-transform' ] = 'translate3d(0, '+ slide_action +'px, 0)';

                                // Do the CSS3 transition
                                base.$slideWrapper.css(properties);

                                base.resetAndUnlock(true);
                            }
                            else
                            {
                                base.$slideWrapper
                                .animate({
                                    "top": slide_action + 'px'
                                }, opt.animationSpeed, base.resetAndUnlock);
                            }
                        }
                        else
                        {
                            base.resetAndUnlock();
                        }
                    }
                    else
                    {
                        var slide_action = base.trsHeight * base.activeSlide < base.numberSlides * base.trsHeight ? '-' + base.trsHeight * base.activeSlide : 0 ;

                        if(doAnimation)
                        {
                            if(base.css3support())
                            {
                                // Get the properties to transition
                                var properties = {};
                                properties[ '-' + base.vendorPrefix + '-transition-duration' ] = opt.animationSpeed + 'ms';
                                properties[ '-' + base.vendorPrefix + '-transform' ] = 'translate3d(0, '+ slide_action +'px, 0)';

                                // Do the CSS3 transition
                                base.$slideWrapper.css(properties);
                                base.resetAndUnlock(true);
                            }
                            else
                            {
                                base.$slideWrapper
                                    .animate({
                                        "top": slide_action + 'px'
                                    }, opt.animationSpeed, base.resetAndUnlock);
                            }
                        }
                        else
                        {
                            base.resetAndUnlock();
                        }
                    }
                    
                }

                /**
                 * Fade
                 */
                if (opt.animation == "fade") 
                {
                    if(base.css3support())
                    {
                        base.$slides.eq(base.activeSlide).css({'z-index': 2});                            

                        // Get the properties to transition
                        var properties = {};
                        properties[ 'opacity' ] = 0;
                        properties[ '-' + base.vendorPrefix + '-transition' ] = 'all ' + opt.animationSpeed + 'ms ease';

                        base.$slides.eq(prevActiveSlide).css(properties);

                        clearTimeout(timeout);
                        timeout = setTimeout(function() {
                            base.$slides.eq(base.activeSlide).css({'z-index': 3});

                            // Get the properties to transition
                            var properties = {};
                            properties[ 'opacity' ] = 1;
                            properties[ 'z-index' ] = 1;
                            properties[ '-' + base.vendorPrefix + '-transition' ] = '';
                            
                            base.$slides.eq(prevActiveSlide).css(properties);
                            base.resetAndUnlock();
                        }, opt.animationSpeed - (opt.animationSpeed * 20 / 100));
                    }
                    else
                    {
                        base.$slides
                            .eq(base.activeSlide)
                            .css({
                                "opacity": 0,
                                "z-index": 3
                            })
                            .animate({
                                "opacity": 1
                            }, opt.animationSpeed, base.resetAndUnlock);
                    }

                    base.setCaptionPosition();
                }
            }
        }

        /**
         * Function: setupSwipeTouch
         */
        base.setupSwipeTouch = function()
        {
            var IMG_WIDTH = opt.animation == "horizontal-slide" ? base.trsWidth : base.trsHeight;
            var currentImg = opt.continousSliding ? base.activeSlideContinous : base.activeSlide;
            var maxImages = base.numberSlides;
            var speed = opt.animationSpeed;

            var imgs;

            var swipeOptions = {
                triggerOnTouchEnd: true,
                triggerOnTouchLeave: true,
                swipeStatus: swipeStatus,
                allowPageScroll: "vertical",
                threshold: 75
            };

            jQuery(function () {
                imgs = opt.continousSliding ? base.$slideWrapper.children().children() : base.$slides;

                imgs.swipe(swipeOptions);
            });


            /**
             * Catch each phase of the swipe.
             * move : we drag the div
             * cancel : we animate back to where we were
             * end : we animate to the next image
             */
            function swipeStatus(event, phase, direction, distance) 
            {
                if(base.locked == false)
                {
                    // reset width and currentImg in case slideshow have been resized
                    IMG_WIDTH = opt.animation == "horizontal-slide" ? base.trsWidth : base.trsHeight;
                    currentImg = opt.continousSliding ? base.activeSlideContinous : base.activeSlide;

                    if (phase == "move") 
                    {
                        var duration = 0;

                        if(opt.animation == "horizontal-slide")
                        {
                            if (direction == "left") {
                                scrollImages((IMG_WIDTH * currentImg) + distance, duration);
                            } else if (direction == "right") {
                                scrollImages((IMG_WIDTH * currentImg) - distance, duration);
                            }
                        }
                        else if(opt.animation == "vertical-slide")
                        {
                            if (direction == "up") {
                                scrollImages((IMG_WIDTH * currentImg) + distance, duration);
                            } else if (direction == "down") {
                                scrollImages((IMG_WIDTH * currentImg) - distance, duration);
                            }
                        }

                    } else if (phase == "cancel") {
                        scrollImages(IMG_WIDTH * currentImg, speed);
                    } else if (phase == "end") {
                        if (direction == "right" || direction == "down") {
                            previousImage();
                        } else if (direction == "left" || direction == "up") {
                            nextImage();
                        }
                    }
                }                
            }

            function previousImage() {
                if(opt.continousSliding)
                {
                    currentImg = currentImg - 1;
                    doShiftAndSwipeScroll('prev');
                }
                else
                {
                    currentImg = Math.max(currentImg - 1, 0);
                    doShiftAndSwipeScroll(currentImg);
                }
            }

            function nextImage() {                
                if(opt.continousSliding)
                {
                    currentImg = currentImg + 1;
                    doShiftAndSwipeScroll('next');
                }
                else
                {
                    currentImg = Math.min(currentImg + 1, maxImages - 1);
                    doShiftAndSwipeScroll(currentImg);
                }
            }

            function doShiftAndSwipeScroll(direction)
            {
                base.shift(direction);
                base.lock();

                scrollImages(IMG_WIDTH * currentImg, speed);
            }

            /**
             * Manually update the position of the imgs on drag
             */
            function scrollImages(distance, duration) 
            {
                var slide_action = (distance < 0 ? "" : "-") + Math.abs(distance).toString();
                var transform_css3, transform_js; 

                if(opt.animation == "horizontal-slide")
                {
                    transform_css3 = 'translate3d('+ slide_action +'px, 0, 0)';
                    transform_js = {"left": slide_action + 'px'};
                }
                else if(opt.animation == "vertical-slide")
                {
                    transform_css3 = 'translate3d(0, '+ slide_action +'px, 0)';
                    transform_js = {"top": slide_action + 'px'};
                }


                if(base.css3support())
                {
                    // Get the properties to transition
                    var properties = {};
                    properties[ '-' + base.vendorPrefix + '-transition-duration' ] = duration + 'ms';
                    properties[ '-' + base.vendorPrefix + '-transform' ] = transform_css3;

                    // Do the CSS3 transition
                    base.$slideWrapper.css(properties);
                    base.resetAndUnlock(true);
                }
                else
                {
                    base.$slideWrapper
                        .animate(transform_js, duration, base.resetAndUnlock);
                }              
            }
        }

        /**
         * Function: resetSlider
         */
        base.resetSlider = function()
        {
            var slide_action;

            base.doLoading(); // do loading
            base.$trsWrapper.parent().width(""); // reset wrapper width
            base.calculateHeightWidth(); // calculate new width & height

            // resize wrapper
            base.$trs.height(base.trsHeight);
            base.$trsWrapper.parent().width(base.trsWidth).height(base.trsHeight);

            base.doResponsiveClass(); // apply responsive class

            // reset active slide & bullet
            base.activeSlide = 0;
            
            base.bulletObj.setActiveBullet();

            // continous & rollback reset attributes
            if(opt.continousSliding)
            {
                base.$slideWrapper.children().children().width(base.trsWidth);
                base.$slideWrapper.children().children().height(base.trsHeight);
                base.$slideWrapper.children().children().children('img').width(base.trsWidth);

                base.activeSlideContinous = base.numberSlides;
                base.continous_count_position = 0;
            }
            else
            {
                base.$slides.width(base.trsWidth);                
                base.$slides.height(base.trsHeight);
                base.$slides.children('img').width(base.trsWidth);

                slide_action = 0;
            }

            // animation based reset attributes
            if(opt.animation == "horizontal-slide")
            {
                if(opt.continousSliding)
                {
                    slide_action = '-' + base.trsWidth * base.activeSlideContinous;

                    base.$slideWrapper.css({
                        'width': base.trsWidth * base.numberSlides * 3 + 'px'
                    });
                }
                else
                {
                    base.$slideWrapper.css({
                        'width': base.trsWidth * base.numberSlides + 'px'
                    });
                }

                // reset slide position
                if(base.css3support())
                {
                    var properties = {};
                    properties['-' + base.vendorPrefix + '-transform'] = 'translate3d('+ slide_action +'px, 0, 0)';
                    properties['margin-left'] = '0px';

                    base.$slideWrapper.css(properties);
                }
                else
                {
                    var properties = {};
                    properties['left'] = slide_action + 'px';
                    properties['margin-left'] = '0px';

                    base.$slideWrapper.css(properties);
                }
            }
            else if(opt.animation == "vertical-slide")
            {
                if(opt.continousSliding)
                {
                    slide_action = '-' + base.trsHeight * base.activeSlideContinous;

                    base.$slideWrapper.css({
                        'height': base.trsHeight * base.numberSlides * 3 + 'px'
                    });
                }
                else
                {
                    base.$slideWrapper.css({
                        'height': opt.height * base.numberSlides + 'px'
                    });
                }

                // reset slide position
                if(base.css3support())
                {
                    var properties = {};
                    properties['-' + base.vendorPrefix + '-transform'] = 'translate3d(0, '+ slide_action +'px, 0)';
                    properties['margin-top'] = '0px';

                    base.$slideWrapper.css(properties);
                }
                else
                {
                    var properties = {};
                    properties['top'] = slide_action + 'px';
                    properties['margin-top'] = '0px';

                    base.$slideWrapper.css(properties);
                }
            }
            else if(opt.animation == "fade")
            {
                base.$slideWrapper.css({"width": base.trsWidth + "px", "height": base.trsHeight + "px"});

                base.$slides.css({
                    "z-index": 1,
                    "width": base.trsWidth + "px",
                    "height": base.trsHeight + "px"
                });

                base.$slides.eq(base.activeSlide).css({"z-index": 3});
            }
            
            // reset slide pagination
            if(opt.pagination == 'text' || opt.pagination == 'image')
            {
                base.bulletObj.generateSlideBullet();
                base.bulletObj.slideBullet('first');
                base.shift(0, true);
            }
        }

        /**
         * Function: doResponsiveClass
         */
        base.doResponsiveClass = function()
        {
            /**
             * Resposive Class
             * - trs-responsive-mobile-small (width <= 369)
             * - trs-responsive-mobile-medium (width <= 499 && width <= trsWidth)
             * - trs-responsive-full
             */

            if(370 <= base.trsWidth && base.trsWidth <= 499) {
                doResponsiveClassStart('trs-responsive-mobile-medium')
            }
            else if(369 >= base.trsWidth ) {
                doResponsiveClassStart('trs-responsive-mobile-small')
            }
            else {
                // Desktop Mode
                doResponsiveClassStart('trs-responsive-full')
            }

            function doResponsiveClassStart(responsiveClass){
                // if it is the first run dont do animation
                if(base.first_run)
                {
                    base.first_run = false
                    base.$trsWrapper.attr('class','trs-wrapper ' + opt.skinClass)
                    base.$trsWrapper.addClass(responsiveClass)
                    return
                }

                if(base.old_responsive_class == responsiveClass) return

                base.old_responsive_class = responsiveClass

                base.$trsWrapper.addClass(responsiveClass)
            }
        }

        /**
         * Function: doLoading
         */
        base.doLoading = function()
        {
            // Do the loading animation
            base.$slideWrapper.hide()

            // Restore & change responsive class
            setTimeout(function() {
                base.$trsWrapper.attr('class','trs-wrapper ' + opt.skinClass);
                base.$slideWrapper.css('display','block');
            }, 1000);
        }
    }


    /**
     * Fluid Responsive Slideshow
     * - default options
     * - initiate each element
     * - initiate return method
     */  
    $.TonjooResponsiveSlideshow.defaults = {
        'animation' : 'horizontal-slide', // horizontal-slide, vertical-slide, fade
        'animationSpeed' : 1000, // how fast animtions are
        'continousSliding' : true, // only works for horizontal-slide and vertical-slide
        'timer' :  true, // true or false to have the timer
        'advanceSpeed' : 3000, // if timer is enabled, time between transitions
        'pauseOnHover' : true, // if you hover pauses the slider
        'startClockOnMouseOut' : false, // if clock should start on MouseOut
        'startClockOnMouseOutAfter' : 800, // how long after MouseOut should the timer start again
        'directionalNav' : 'autohide', // autohide, show, none
        'directionalNavShowOpacity' : '0.9', // from 0 to 1
        'directionalNavHideOpacity' : '0.1', // from 0 to 1
        'pagination' : 'bullet', // bullet, text, image, none
        'paginationContent' : [], // can be text, image, or something
        'paginationContentWidth': 200, // pagination content width in pixel
        'skinClass' : 'trs-skin-default', // default: trs-skin-default
        'width' : 500, // slideshow width (actually max-width)
        'height' : 270, // slideshow height (will auto generate if width changed by browser size)
        'jsOnly' : false // for development testing purpose
    };

    $.fn.TonjooResponsiveSlideshow = function(options) 
    {
        var base = this;
        var opt = $.extend({}, $.TonjooResponsiveSlideshow.defaults, options);
        var plugin = new $.TonjooResponsiveSlideshow(base, opt);

        // manual shift
        base.doShift = function(value)
        {
            plugin.stopSliderLock();
            plugin.shift(value, true);
        }

        // external pagination shift
        var paginationClass = opt.paginationExternalClass;

        if(paginationClass != "" && $('.' + paginationClass).length)
        {
            var paginationClass = opt.paginationExternalClass;

            $('.' + paginationClass).click(function(){
                base.doShift($('.' + paginationClass).index(this));
            })
        }
        
        base.each(function(){
            plugin.initialize();
        });
        
        return base;
    };

})(jQuery);